<?php

chdir('../../');
require_once('api/Simpla.php');
$simpla = new Simpla();

/*
 * 
 * Глобальный массив настроек платежа.
 * 
 */

$payment_settings = array();

function get_constant($name) {
    global $payment_settings;
    return isset($payment_settings[$name]) ? $payment_settings[$name] : NULL;
}

/*
 * 
 * Преобразует число к требуемому платежкой формату.
 * 
 */

function to_float($sum) {
    $sum2 = round(floatval($sum), 2);
    $sum = sprintf("%01.2f", $sum2);
    if (substr($sum, -1) == "0") {
        $sum = sprintf("%01.1f", $sum2);
    }
    return $sum;
}

//функция выдает ответ для сервиса onpay в формате XML на check запрос 
function answer($type, $code, $pay_for, $order_amount, $order_currency, $text) {
    $md5 = strtoupper(md5("{$type};{$pay_for};{$order_amount};{$order_currency};{$code};" . get_constant('secret_key')));
    return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<result>\n<code>{$code}</code>\n<pay_for>{$pay_for}</pay_for>\n<comment>{$text}</comment>\n<md5>{$md5}</md5>\n</result>";
}

//функция выдает ответ для сервиса onpay в формате XML на pay запрос 
function answerpay($type, $code, $pay_for, $order_amount, $order_currency, $text, $onpay_id) {
    $md5 = strtoupper(md5("{$type};{$pay_for};{$onpay_id};{$pay_for};{$order_amount};{$order_currency};{$code};" . get_constant('secret_key')));
    return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<result>\n<code>{$code}</code>\n<comment>{$text}</comment>\n<onpay_id>{$onpay_id}</onpay_id>\n<pay_for>{$pay_for}</pay_for>\n<order_id>{$pay_for}</order_id>\n<md5>{$md5}</md5>\n</result>";
}

/*
 * 
 * Обрабатывает обращения платежной системы к API сайта.
 * 
 */

function callback() {
    global $simpla;

    if ($simpla->request->method('post')) {

        // Принимаем данные.
        $type = $simpla->request->post('type', 'string'); // тип платежа
        $pay_for = $simpla->request->post('pay_for', 'integer'); // номер заказа
        $order_amount = $simpla->request->post('order_amount'); // стоисоть заказа
        $order_currency = $simpla->request->post('order_currency'); // валюта заказа
        $signature = strtoupper($simpla->request->post('md5', 'string')); // подпись магазина - защита от подделки запроса

        $order = $simpla->orders->get_order($pay_for);

        // CHECK запрос - позволяет проверить разрешил ли магазин указанный платеж.
        if ($type == 'check') {

            $error = '';
            if (!empty($order)) {
                global $payment_settings;
                $payment_method = $simpla->payment->get_payment_method($order->payment_method_id);
                $payment_settings = $simpla->payment->get_payment_settings($payment_method->id);

                $currency = $simpla->money->get_currency($payment_method->currency_id);
                $purchases = $simpla->orders->get_purchases(array('order_id' => intval($order->id)));

                $md5 = strtoupper(md5("$type;$pay_for;$order_amount;$order_currency;" . get_constant('secret_key')));
                if ($md5 != $signature) {
                    return answer($type, '7', $pay_for, $order_amount, $order_currency, 'Incorrect cheksum.');
                }
                $order_total_price = to_float($simpla->money->convert($order->total_price, $payment_method->currency_id, false));

                if ($order->paid) {
                    return answer($type, '2', $pay_for, $order_amount, $order_currency, 'This order have already been paid.');
                    ;
                } elseif ($order_currency != $currency->code && $order_currency != 'TST')
                    $error = 'Incorrect currency code.';
                elseif ($order_amount != $order_total_price)
                    $error = 'Incorrect order price.';
            } else {
                $error = 'There is no order_for simpla ID';
            }

            if (!empty($error)) {
                return answer($type, '3', $pay_for, $order_amount, $order_currency, $error);
            }

            foreach ($purchases as $purchase) {
                $variant = $simpla->variants->get_variant(intval($purchase->variant_id));
                if (empty($variant) || (!$variant->infinity && $variant->stock < $purchase->amount)) {
                    return answer($type, '2', $pay_for, $order_amount, $order_currency, 'There is not enough products on the stock.');
                }
            }

            echo answer($type, '0', $pay_for, $order_amount, $order_currency, 'OK!');
        }

        // PAY запрос - уведомление магазина о поступившем платеже.
        elseif ($type == 'pay') {

            $onpay_id = $simpla->request->post('onpay_id', 'integer');

            $error = '';
            if (!empty($order)) {
                global $payment_settings;
                $payment_method = $simpla->payment->get_payment_method($order->payment_method_id);
                $payment_settings = $simpla->payment->get_payment_settings($payment_method->id);

                $currency = $simpla->money->get_currency($payment_method->currency_id);
                $purchases = $simpla->orders->get_purchases(array('order_id' => intval($order->id)));

                $md5 = strtoupper(md5("$type;$pay_for;$onpay_id;$order_amount;$order_currency;" . get_constant('secret_key')));
                if ($md5 != $signature) {
                    return answerpay($type, '7', $pay_for, $order_amount, $order_currency, 'Incorrect cheksum.', $onpay_id);
                }

                $simpla->orders->update_order(intval($order->id), array('paid' => 1));
                $simpla->orders->close(intval($order->id));
                $simpla->notify->email_order_user(intval($order->id));
                $simpla->notify->email_order_admin(intval($order->id));

                return answerpay($type, '0', $pay_for, $order_amount, $order_currency, 'OK!', $onpay_id);
            } else {
                $error = 'There is no order_for simpla ID';
            }

            if (!empty($error)) {
                return answerpay($type, '3', $pay_for, $order_amount, $order_currency, $error, $onpay_id);
            }
        }
    }
}

try {
    $result = callback();
    if ($result) {
        echo $result;
    }
} catch (Exception $e) {
    echo($e->getTraceAsString());
}
